package org.vraptor;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.vraptor.component.ComponentNotFoundException;
import org.vraptor.component.LogicNotFoundException;
import org.vraptor.config.ConfigException;
import org.vraptor.core.Controller;
import org.vraptor.core.ControllerFactory;
import org.vraptor.core.WebRequest;
import org.vraptor.interceptor.InterceptorInstantiationException;
import org.vraptor.url.InvalidURLException;
import org.vraptor.view.ViewException;

/**
 * VRaptor servlet contains a vraptor controller capable of answering web
 * requests.
 *
 * @author Guilherme Silveira
 */
public class VRaptorServlet extends HttpServlet {

	private static final Logger LOG = Logger.getLogger(VRaptorServlet.class);

	/** serialversion uid */
	private static final long serialVersionUID = 1084551538406918486L;

	private final static String CHECK_THE_LOGS_MESAGE = " For further information, check the log files for the root cause."
			+ " Enable VRaptor DEBUG/TRACE messages in log4J if necessary.";

	private Controller controller;

	private boolean initialized;

	public VRaptorServlet(Controller controller) {
		this.controller = controller;
		this.initialized = true;
	}

	public VRaptorServlet() {
	}

	@Override
	public synchronized void init() throws ServletException {
		if (!this.initialized) {
			super.init();
			try {
				this.controller = new ControllerFactory().configure(getServletContext());
			} catch (ConfigException e) {
				throw new ServletException("Unable to configure vraptor2", e);
			}
			this.initialized = true;
		}
	}

	@Override
	protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException,
			IOException {

		if (LOG.isDebugEnabled()) {
			LOG.debug("new request to " + request.getRequestURI());
		}
		try {
			controller.execute(new WebRequest(request, response, getServletContext()));
		} catch (LogicNotFoundException e) {
			LOG.debug("Logic not found", e);
			sendError(response, 404, "URL not found, no logic matches for this component.", e);
		} catch (InvalidURLException ex) {
			throw new ServletException("Unable to deal with selected url", ex);
		} catch (ComponentNotFoundException e) {
			LOG.debug("Component not found", e);
			sendError(response, 404, "URL not found, no registered component matches.", e);
		} catch (InterceptorInstantiationException e) {
			LOG.debug("Instatiating an interceptor resulted an exception.", e);
			throw new ServletException(e.getMessage(), e);
		} catch (ViewException ex) {
			LOG.debug("Rendering view resulted an exception.", ex);
			if (ex.getCause() == null) {
				throw new ServletException(ex.getMessage(), ex);
			} else {
				throw new ServletException(ex.getCause().getMessage(), ex.getCause());
			}
		} catch (LogicException ex) {
			LOG.debug("Logic exception.", ex);

			if (ex.getCause() == null) {
				throw new ServletException(ex.getMessage(), ex);
			} else {
				throw new ServletException(ex.getCause().getMessage(), ex.getCause());
			}
		} catch (Exception e) {
			LOG.debug("VRaptor unexpected exception", e);
			throw new ServletException(e);
		}

	}

	private void sendError(HttpServletResponse response, int error, String message, Exception e) throws IOException {
		StringBuilder finalMessage = new StringBuilder(message);
		finalMessage.append(" Cause: ");
		finalMessage.append(e.toString());
		finalMessage.append(". ");
		finalMessage.append(CHECK_THE_LOGS_MESAGE);
		response.sendError(error, finalMessage.toString());

	}

}
