package org.vraptor.component;

import java.lang.annotation.Annotation;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import org.apache.log4j.Logger;
import org.vraptor.LogicException;
import org.vraptor.LogicRequest;
import org.vraptor.annotations.Logic;
import org.vraptor.annotations.Parameter;
import org.vraptor.annotations.Viewless;
import org.vraptor.reflection.StringUtil;
import org.vraptor.validator.BasicValidationErrors;
import org.vraptor.validator.UnstableValidationException;
import org.vraptor.validator.ValidationErrors;

/**
 * Represents a logic method.
 *
 * @author Guilherme Silveira
 */
public class DefaultLogicMethod implements LogicMethod {

	private static final Logger LOG = Logger.getLogger(DefaultLogicMethod.class);

	private final String name;

	private final Method method;

	private final boolean shouldRedirect;

	private final Method validateMethod;

	private final List<MethodParameter> parameters;

	/**
	 * Constructs it based on the logics name and method
	 *
	 * @param method
	 *            method
	 */
	public DefaultLogicMethod(String name, Method method, Method validateMethod) {
		this.name = name;
		this.method = method;
		this.shouldRedirect = !this.method.isAnnotationPresent(Viewless.class);
		this.validateMethod = validateMethod;

		this.parameters = loadParameters(method);

		if (LOG.isDebugEnabled()) {
			// TODO trace
			LOG.debug("Logic method " + method.getName() + " was read with validation " + this.validateMethod
					+ " and parameters " + parameters);
		}
	}

	public DefaultLogicMethod(String name, Method method) {
		this(name, method, null);
	}

	private List<MethodParameter> loadParameters(Method method) {
		Class<?>[] params = method.getParameterTypes();
		Type[] generic = method.getGenericParameterTypes();
		Annotation[][] annotations = method.getParameterAnnotations();
		List<MethodParameter> list = new ArrayList<MethodParameter>();

		Logic logic = method.getAnnotation(Logic.class);
		String[] parameterNames = logic != null ? logic.parameters() : null;
		if (parameterNames != null && parameterNames.length != 0
				&& parameterNames.length != method.getParameterTypes().length) {
			throw new IllegalArgumentException("The number of parameters at @Logic does "
					+ "not match the number of this method arguments");
		}

		for (int i = 0; i < params.length; i++) {
			Class<?> param = params[i];
			Parameter parameterAnnotation = searchParameterAnnotation(annotations, i);
			String key;

			if (parameterAnnotation != null && !parameterAnnotation.key().equals("")) {
				// using @Parameter key name
				key = parameterAnnotation.key();
			} else {
				// using @Logic(parameters={..} value
				if (parameterNames != null && parameterNames.length != 0) {
					key = parameterNames[i];
				} else {
					key = StringUtil.classNameToInstanceName(param.getSimpleName());
				}
			}

			list.add(new MethodParameter(param, generic[i], i, key));
		}
		return list;
	}

	private Parameter searchParameterAnnotation(Annotation[][] annotations, int i) {
		for (Annotation a : annotations[i]) {
			if (a.annotationType().equals(Parameter.class)) {
				return (Parameter) a;
			}
		}
		return null;
	}

	public String getName() {
		return this.name;
	}

	public String execute(Object component, LogicRequest context, Object[] params) throws LogicException {
		try {
			Object result = method.invoke(component, params);
			return result == null ? "ok" : result.toString();
		} catch (IllegalArgumentException e) {
			throw new LogicException(e);
		} catch (IllegalAccessException e) {
			throw new LogicException(e);
		} catch (InvocationTargetException e) {
			throw new LogicException(e.getCause());
		}
	}

	public ValidationErrors validate(Object component, LogicRequest context, ResourceBundle bundle, Object[] params)
			throws UnstableValidationException {
		ValidationErrors errors = new BasicValidationErrors();
		if (validateMethod != null) {
			try {
				validateMethod.invoke(component, addValidationErrorsToArray(errors, params));
			} catch (IllegalArgumentException e) {
				LOG.error("Nasty validation method has thrown an exception!", e);
				throw new UnstableValidationException("Error during validation process", e);
			} catch (IllegalAccessException e) {
				LOG.error("Nasty validation method has thrown an exception!", e);
				throw new UnstableValidationException("Error during validation process", e);
			} catch (InvocationTargetException e) {
				LOG.error("Nasty validation method has thrown an exception!", e);
				throw new UnstableValidationException("Error during validation process", e);
			}
		}
		return errors;
	}

	private Object[] addValidationErrorsToArray(ValidationErrors errors, Object[] params) {

		Object[] types = new Object[params.length+1];
		types[0] = errors;

		int i = 1;

		for(Object param : params) {
			types[i++] = param;
		}
		return types;
	}

	public boolean shouldRedirect() {
		return this.shouldRedirect;
	}

	public Method getMetadata() {
		return method;
	}

	public List<MethodParameter> getParameters() {
		return parameters;
	}

}
