package org.vraptor.webapp;

import java.lang.reflect.Modifier;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import org.apache.log4j.Logger;
import org.vraptor.component.ComponentManager;
import org.vraptor.component.ComponentNotFoundException;
import org.vraptor.component.ComponentType;
import org.vraptor.component.ComponentTypeFactory;
import org.vraptor.component.InvalidComponentException;
import org.vraptor.component.LogicMethod;
import org.vraptor.component.LogicNotFoundException;
import org.vraptor.url.RequestInfo;

/**
 * A simple implementation of a component manager.
 * 
 * @author Guilherme Silveira
 */
public class DefaultComponentManager implements ComponentManager {

	private static final Logger LOG = Logger.getLogger(DefaultComponentManager.class);

	private final ConcurrentMap<String, ConcurrentMap<String, ComponentType>> components = new ConcurrentHashMap<String, ConcurrentMap<String, ComponentType>>();

	private final ComponentTypeFactory componentClassFactory = new ComponentTypeFactory();

	public boolean register(String typeClazz) throws InvalidComponentException {
		try {
			Class<?> type = Class.forName(typeClazz);
			if (Modifier.isPublic(type.getModifiers())) {
				ComponentType component = this.componentClassFactory.getComponentType(type);
				register(component);
				return true;
			} else {
				LOG.warn("Ignoring non public class " + typeClazz);
				return false;
			}
		} catch (LogicNotFoundException e) {
			throw new InvalidComponentException("Unable to logic for type " + typeClazz, e);
		} catch (ClassNotFoundException e) {
			throw new InvalidComponentException("Unable to find type " + typeClazz, e);
		}
	}

	public ComponentType getComponent(RequestInfo info) throws ComponentNotFoundException, LogicNotFoundException {
		if (!this.components.containsKey(info.getComponentName())) {
			throw new ComponentNotFoundException(String.format("Component for %s not found", info.getComponentName()));
		}
		if (!this.components.get(info.getComponentName()).containsKey(info.getLogicName())) {
			throw new LogicNotFoundException(String.format("Logic for %s not found in component %s", info
					.getLogicName(), info.getComponentName()));
		}
		return this.components.get(info.getComponentName()).get(info.getLogicName());
	}

	public Set<ComponentType> getComponents() {
		HashSet<ComponentType> components = new HashSet<ComponentType>();
		for (Map<String, ComponentType> map : this.components.values()) {
			components.addAll(map.values());
		}
		return components;
	}

	public void register(ComponentType type) throws LogicNotFoundException {
		LOG.debug(String.format("Registering component %s as %s", type.getComponentClass(), type.getName()));
		if (!this.components.containsKey(type.getName())) {
			this.components.put(type.getName(), new ConcurrentHashMap<String, ComponentType>());
		}
		for (LogicMethod logic : type.getLogics()) {
			this.components.get(type.getName()).put(logic.getName(), type);
		}
	}

}
